//Paquete al que poertenece la clase.
package CocheManualAutomatico;

/**
 * Clase que crear objetos Coche, y que permite aumentar y disminuir su velocidad y marcha (slo clases hijas),
 * as como obtener sus valores de velocidad y marcha y obtener el estado actual de cada objeto mediante 
 * una cadena de texto.
 * 
 * @exception MatriculaInvalida error que lanzar el constructor si se pasa un parmetro no vlido.
 * @author Prof.
 * @version 1.0
 *
 */
public class Coche {
	// Atributos de clase:
	private static int cochesFabricados = 0; // Contador que indicar cuantos Coches se crean.
	protected final static int MAX_MARCHA = 4; // Valor mximo de marcha que podr usarse.

	// Atributos de cada objeto Coche:
	private int numeroBastidor; // Nmero que ser nico para cada Coche.
	private String matricula; // Cadena de caracteres con la mtricula individual de cada Coche.
	protected int velocidad; // Velocidad que lleva cada Coche.
	protected int marcha; // Marcha actual que lleva cada Coche.

	/**
	 * Constructor de la clase Coche que recibe como parrmetro una cadena con la matrcula a asiganr al Coche
	 * y comprueba si su formato es el correcto segn lo establecido, en cuyo crear el Coche, si no
	 * es correcto el objeto no se crear.
	 * 
	 * @author Prof.
	 * @param matricula Cadena con la matrcula.
	 * @throws MatriculaInvalida Si la matrcula pasada es incorrecta.
	 * @version 1.0
	 */
	public Coche(String matricula) throws  MatriculaInvalida {
		// Se comprueba que la cadena pasada cumple empezar por 4 dgitos del 0 al 9, seguido de un guin y de 3 letras quitando las vocales
		if (matricula.matches("[0-9]{4}-[A-Z&&[^AEIOU]]{3}")) {
			// Si la matrcula es correcta se inicializan todos los atributos del coche que se est creando.
			this.matricula = matricula;
			// Por cada coche que se crea correctamente se aumenta la variable de clase coche en una unidad.
			cochesFabricados++;
			// El nuevo coche creado tendr como bastidor el nmero correspondiente del contador de Coches creados.
			numeroBastidor = cochesFabricados;
			// Velocidad y Marcha se establecen a 0 representando que el coche est parado/apagado.
			velocidad = 0;
			marcha = 0;
		} else {
			// Si no es posible crear el coche porque la matrcula pasada es erronea se imprime el mensaje de error.
			throw new MatriculaInvalida();
		}
	}

	/**
	 * Mtodo que devuelve el valor de la marcha que lleva actualmente el coche.
	 * 
	 * @author Prof.
	 * @return Entero con la marcha actual.
	 * @version 1.0
	 */
	public int getMarcha() {
		// Se devuelve la marcha.
		return marcha;
	}

	/**
	 * Mtodo que devuelve el valor de la velocidad que lleva actualmente el coche.
	 * 
	 * @author Prof.
	 * @return Entero con la velocidad actual.
	 * @version 1.0
	 */
	public int getVelocidad() {
		// se devuelve la velocidad.
		return velocidad;
	}

	/**
	 * El mtodo comprueba el valor de la velocidad introducida como parmetro y si es positivo aumenta la velocidad
	 * en la cantidad indicada si es negativo llama al mtodo frenar.
	 * 
	 * @author Prof.
	 * @param velocidad valor con la cantidad a aumnetar la velocidad.
	 * @version 1.0
	 */
	public void acelerar(int velocidad) {
		if (velocidad >= 0) {
			// Si la velocidad pasada es positiva se aumenta la variable velocidad con su valor + el valor pasado como parmetro.
			velocidad += velocidad;
		} else {
			// Si la valor pasado es negativo se llama a frenar() para que disminuya el valor dela velocidad.
			frenar(-(velocidad));
		}

	}

	/**
	 * El mtodo comprueba el valor de la velocidad introducida como parmetro y si es negativo disminuye la velocidad
	 * en la cantidad indicada si es positiva llama al mtodo acelerar.
	 * 
	 * @author Prof.
	 * @param velocidad valor con la cantidad a aumnetar la velocidad.
	 * @version 1.0
	 */
	public void frenar(int velocidad) {
		if (velocidad >= 0) {
			// Si la velocidad pasada es negativa se disminuye la variable velocidad con su valor + (- el valor pasado como parmetro).
			velocidad += velocidad;
		} else {
			// Si la valor pasado es positivo se llama a acelerar() para que aumente el valor dela velocidad.
			acelerar(-(velocidad));
		}
	}

	/**
	 * Mtodo que cambia la marcha que lleva actualmente el coche. No realiza comprobaciones
	 * por ello slo puede ser invocado desde las subclases que hereden Coche.
	 * 
	 * @author Prof.
	 * @param marca valor con la marcha que se quiere poner.
	 * @version 1.0
	 */
	protected void cambiarMarcha(int marcha) {
		// No se realizan comprobaciones ya que el valor pasado ser controlado en las clases hijas.
		this.marcha = marcha;
	}

	/**
	 * Mtodo para representar el estado de los objetos tipo Coche.
	 * 
	 * @return Cadena con la informacin del objeto.
	 */
	@Override
	public String toString() {
		// Se devuelve la cadena con la informacin de los atributos.
		return "Coche [Nmero de Bastidor = " + numeroBastidor + ", Matrcula = " + matricula + ", Velocidad = " + velocidad + "km/h, Marcha=" + marcha + "] - ";
	}
}